/******************************************************************************
 *
 *     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 *     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
 *     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
 *     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
 *     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
 *     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 *     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 *     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
 *     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 *     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 *     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 *     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *     FOR A PARTICULAR PURPOSE.
 *     
 *     (c) Copyright 2005 Xilinx, Inc.
 *     All rights reserved.
 *
 ******************************************************************************/
 
//----------------------------------------------------------------------------
// Filename:     initializeDisplay.c
// 
// Description:	Setup the initial display of the mandelbrot demo on VGA screen.
//				Titles of the screen, graph axes etc are drawn here.
// Design Notes:
//   
//-----------------------------------------------------------------------------
// Xilinx APD
// History:
// 			Created - 07/2005 - KRS
//
//-----------------------------------------------------------------------------

#include "mand.h"
#include "bootload_basicgraphics.h"
#include "xtft_main.h"
#include "co.h"
#include "sleep.h"

#define NO_OF_FLASHES 10

//Draws initial display of Statistics and Graph on TFT screen
void initializeDisplay()
{
	uint32 i;
	
    /////////////////////TITLE QUADRANT///////////////////////////
	//Demo title	
	//Usage: XTft_SetPos(pointer, x-coordinate, y-coordinate);
	XTft_SetPos(&Tft, 50, 3);
	//Usage: XTft_WriteColorString(pointer, string_to_plot, text_color, background_color);
	XTft_WriteColorString(&Tft, "APU INTERFACE AND C-TO-HDL", 0x00FFFFFF, 0x00000000);

	XTft_SetPos(&Tft, 138, 18);
	XTft_WriteColorString(&Tft, "-VS-", 0x00FFFFFF, 0x00000000);

	XTft_SetPos(&Tft, 125, 33);
	XTft_WriteColorString(&Tft, "PowerPC", 0x00FFFFFF, 0x00000000);

	//Usage: XTft_DrawLine(pointer, x1, y1, x2, y2, color);
	XTft_DrawLine(&Tft, 15, 50, 305, 50, 0xFFFFFF);

	//Descriptive text
	XTft_SetPos(&Tft, 10, 65);
	XTft_WriteColorString(&Tft, "The APU interface and Impulse C-to-HDL", 0x00FFFFFF, 0x00000000);
	XTft_SetPos(&Tft, 10, 80);
	XTft_WriteColorString(&Tft, "tools provide a significant speedup in ", 0x00FFFFFF, 0x00000000);
	XTft_SetPos(&Tft, 10, 95);
	XTft_WriteColorString(&Tft, "system performance for fractal image", 0x00FFFFFF, 0x00000000);
	XTft_SetPos(&Tft, 10, 110);
	XTft_WriteColorString(&Tft, "generation.", 0x00FFFFFF, 0x00000000);

	//Time Statistics 
	XTft_SetPos(&Tft, 96, 155);
	XTft_WriteColorString(&Tft, "COMPLETION  TIME", 0x00FFFFFF, 0x00000000);
	XTft_DrawLine(&Tft, 110, 168, 210, 168, 0xFFFFFF);

	XTft_SetPos(&Tft, 10, 175);
	XTft_WriteColorString(&Tft, "PowerPC        =", 0xFF0000, 0);

	XTft_SetPos(&Tft, 10, 190);
	XTft_WriteColorString(&Tft, "APU + C-to-HDL =", 0xFF00, 0);

	XTft_SetPos(&Tft, 10, 220);
	XTft_WriteColorString(&Tft, "Speedup        =", 0xFF00, 0);
    ///////////////////END TITLE QUADRANT/////////////////////////


    /////////////////////GRAPH QUADRANT///////////////////////////
	//Heading
	XTft_SetPos(&Tft, 118, 245);
	XTft_WriteColorString(&Tft, "SPEED GRAPH", 0xFFFFFF, 0);	
	XTft_DrawLine(&Tft, 110, 258, 210, 258, 0xFFFFFF);

	//Vertical axis
	XTft_DrawLine(&Tft, 60, 280, 60, 440, 0xFFFFFF);

	//Horizontal axis
	XTft_DrawLine(&Tft, 60, 440, 305, 440, 0xFFFFFF);

	//Axes' titles
	XTft_SetPos(&Tft, 10, 265);
	XTft_WriteColorString(&Tft, "Lines/sec", 0xFFFFFF, 0);
	XTft_SetPos(&Tft, 15, 280);
	XTft_WriteColorString(&Tft, "3800", 0xFFFFFF, 0);
	XTft_SetPos(&Tft, 25, 431);
	XTft_WriteColorString(&Tft, "<25", 0xFFFFFF, 0);

	XTft_SetPos(&Tft, 150, 445);
	XTft_WriteColorString(&Tft, "Line No.", 0xFFFFFF, 0);	

	XTft_SetPos(&Tft, 60, 445);
	XTft_WriteColorString(&Tft, "0", 0xFFFFFF, 0);

	XTft_SetPos(&Tft, 290, 445);
	XTft_WriteColorString(&Tft, "240", 0xFFFFFF, 0);

	//Horizontal axis markers
	for (i = 65 ; i <= 305 ; i += 5)
		XTft_DrawLine(&Tft, i, 438, i, 442, 0xFFFFFF);

	//Vertical axis markers	
	for (i = 280 ; i < 440 ; i += 5)
		XTft_DrawLine(&Tft, 58, i, 62, i, 0xFFFFFF);
    ////////////////////END GRAPH QUADRANT////////////////////////
}

//Flash the word 'APU + C-to-HDL' on screen to alert user to APU + C-2-HDL drawing of Mandelbrot
void flashApu(void)
{
	uint32 i;
	//Flash 'NO_OF_FLASHES' times
	for (i = 0 ; i < NO_OF_FLASHES ; ++i)
	{
		//Write black on green background
		XTft_SetPos(&Tft, 10, 190);
		XTft_WriteColorString(&Tft, "APU + C-to-HDL", 0, 0xFF00);
		usleep(100000);

		//Write green on black background
		XTft_SetPos(&Tft, 10, 190);
		XTft_WriteColorString(&Tft, "APU + C-to-HDL", 0xFF00, 0);
		usleep(100000);
	}
}

//Flash the word 'PowerPC' on screen to alert user to SOFTWARE-ONLY(PowerPC) drawing of Mandelbrot
void flashPowerpc(void)
{
	uint32 i;

	//Flash 'NO_OF_FLASHES' times
	for (i = 0 ; i < NO_OF_FLASHES ; ++i)
	{
		//Write black on red background
		XTft_SetPos(&Tft, 10, 175);
		XTft_WriteColorString(&Tft, "PowerPC", 0, 0xFF0000);
		usleep(100000);
		
		//Write red on black background
		XTft_SetPos(&Tft, 10, 175);
		XTft_WriteColorString(&Tft, "PowerPC", 0xFF0000, 0);
		usleep(100000);
	}
}

