/******************************************************************************
 *
 *     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 *     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
 *     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
 *     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
 *     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
 *     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 *     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 *     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
 *     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 *     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 *     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 *     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *     FOR A PARTICULAR PURPOSE.
 *     
 *     (c) Copyright 2005 Xilinx, Inc.
 *     All rights reserved.
 *
 ******************************************************************************/

//----------------------------------------------------------------------------
// Filename:		mand_sw_only.c
// 
// Description:		The software-only version of the Mandelbrot image generation
//					algorithm. 
//
// Design Notes:
//   
//-----------------------------------------------------------------------------
// Xilinx APD
// History:
// July 2005 - Created - Kunal Shenoy
//
//-----------------------------------------------------------------------------

#ifdef WIN32
	#include <windows.h>
	#include <sys/types.h>
	#include <sys/stat.h>
#else  // ! WIN32
	#include <sys/stat.h>
	#include <unistd.h>
#endif

#include <stdio.h>
#include "co.h"
#include "mand.h"

#ifdef IMPULSE_C_TARGET
	#include "bootload_basicgraphics.h"
	#include "xtft_main.h"
	#include "stop_watch.h"
#else 
	//Bitmap file storing SW generated Mandelbrot during desktop simulations
	extern FILE * outfileSW;
#endif

//SW-only version of mandelbrot image generation code
void fix_mandelbrot(int32 xmax, int32 xmin, int32 ymax, int32 ymin, uint32 dx, uint32 dy, uint32 hdx, uint32 hdy) 
{
    uint32 c_imag,c_real;	//complex number added in calculations
	uint32 j, i, k;			//loop variables
	int R, G, B;			//pixel colors
    uint32 result, tmp;		//calculation variables
    uint32 z_real, z_imag;	//complex number in calculation
    co_int24 pixelValue;	//value of pixel
	co_uint32 two, four;	//holds fixed point representations of integers
    unsigned int pixelCount = 0;	//number of pixels counted per line
#ifndef IMPULSE_C_TARGET
	co_uint24 byteValue;	//stores value to be written to bitmap file
#else
	uint32 graph_x = 61;	//starting x coordinate of speed graph
	uint32 time_line;		//y co-ordinate of speed graph to be plotted
#endif

	//constant integers assigned in fixed point format
	//#define FXCONST(a) FXCONST32(a,FRACBITS)
	//#define FXCONST32(a,DW) ((uint32)((a)<<DW))
	two = FXCONST(2);
	four = FXCONST(4);

    c_imag=ymax;
	//for each horizontal line
    for (j = 0; j < YSIZE; j++) 
	{
		//keep track of time consumed per line
#ifdef IMPULSE_C_TARGET
        stop_watch(1,1);
#endif
		c_real=xmin;
		//for each pixel of the horizontal line
        for (i = 0; i < XSIZE; i++) 
		{
            z_real = z_imag = 0;
			//compute pixel value
            k = 0;
            do 
			{
				//Square the complex number, add constant and check
				//magnitude
				tmp = z_real;
				z_real = FXMUL(z_real, z_real);
				z_real = FXSUB(z_real, FXMUL(z_imag, z_imag));
				z_real = FXADD(z_real, c_real);
				z_imag = FXMUL(tmp, z_imag);
				z_imag = FXMUL(two,z_imag);
				z_imag = FXADD(z_imag, c_imag);
				tmp=FXMUL(z_real, z_real);
				result = FXADD(tmp, FXMUL(z_imag, z_imag));
				k++;
            } while (result < (4*(1<<FRACBITS)) && k < MAX_ITERATIONS);
			//determine color of pixel
            B = G = R = 0;
            if (k != MAX_ITERATIONS) 
			{
                R = k > 255 ? 255 : k;
            }
			pixelValue = ((R<<16) | (G<<8) | B) & 0x00FFFFFF;

#ifdef IMPULSE_C_TARGET //If running on FPGA
			XTft_SetPixel(&Tft, i+320, j, pixelValue); //Plot on first quadrant of VGA screen
#else		//If running on desktop simulation mode
			//write pixel color to bitmap file
			if (pixelCount % XSIZE == 0) 
			{
				printf("\rLine %d",pixelCount/XSIZE);
				fflush(stdout);
			}
			byteValue = 0;
			byteValue = (unsigned int) pixelValue & REDMASK;
			putc(byteValue, outfileSW);
			byteValue = (unsigned int) pixelValue & GREENMASK;
			byteValue >>= 8;
			putc(byteValue, outfileSW);
			byteValue = (unsigned int) pixelValue & BLUEMASK;
			byteValue >>= 16;
			putc(byteValue, outfileSW);
#endif			
            pixelCount++;
            c_real=FXADD(c_real,dx);
        }
        c_imag = FXSUB(c_imag,dy);

//If running on FPGA calculate time per line and plot on screen
#ifdef IMPULSE_C_TARGET
		time_line = 435 - (40000 / stop_watch(4,1));
		XTft_SetPixel(&Tft, graph_x++, time_line, 0xFF0000);
#endif
    }
}
