/******************************************************************************
 *
 *     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 *     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
 *     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
 *     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
 *     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
 *     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 *     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 *     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
 *     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 *     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 *     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 *     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *     FOR A PARTICULAR PURPOSE.
 *     
 *     (c) Copyright 2005 Xilinx, Inc.
 *     All rights reserved.
 *
 ******************************************************************************/

//----------------------------------------------------------------------------
// Filename:     stopwatch.c
// 
// Description:    
// Stopwatch routines for code timing and comparison between software-only
//	and hardware-software implementations
// Design Notes:
//   
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Xilinx APD
// History:-----------------------------
//			07/2005 - Modified - KRS - Added timerNo parameter to have dual timers
//----------------------------------------------------------------------------

#define FLOAT_LIB

#include "stdio.h"
#include "xtime_l.h"
#include "xparameters.h"

// stop_watch routine
// supports split timming
// call with mode = 
//		0 reset timer (must always call first if using start)
//		1 reset and start timer
//		2 start/restart timer
//		3 pause/stop timer
//		4 stop timer and display total time
//		5 display total time
// returns current total time in microseconds

int stop_watch(int mode, int timerNo)
{
// tStart is in system clock units
// tMark is in nanoseconds
 	XTime tCur;
	static XTime tStart0, tTotal0;
	static XTime tStart1, tTotal1;
	static XTime *tStart;
	static XTime *tTotal;
	int result;

	//pick one of the two timer variables based on timerNo
	tStart = (timerNo == 1) ? &tStart1 : &tStart0;
	tTotal = (timerNo == 1) ? &tTotal1 : &tTotal0;

	// capture current time
	XTime_GetTime(&tCur);

	switch (mode)
	{
		case 0: // initialize timer
			*tTotal = 0;
			break;
		case 1: // initialize and start timer
			*tTotal = 0;
		case 2: // start or restart timer
			// save current time for stopwatch
			XTime_GetTime(tStart);
			break;
		case 3: // pause timer
			// compute total time up to now in processor cycles and add to previous value
			*tTotal += (tCur - *tStart);
			break;
		case 4: // stop timer and display time
			// compute total time up to now in processor cycles and add to previous value
			*tTotal += (tCur - *tStart);
			break;
		case 5: // display total time
			*tTotal += (tCur - *tStart);
			// compute total time in microseconds
			result = (*tTotal * (XTime) 1000000) / (XTime) XPAR_CPU_PPC405_CORE_CLOCK_FREQ_HZ;
			// output scaled answer
			if (result < 100000)
				xil_printf ("  Time (microsec): %d \n\r",result);
			else
			{
				result = (result +500) / 1000; // scale by 1000 with rounding
				if (result < 100000)
					xil_printf ("  Time (millisec): %d \n\r",result);
				else
				{
					result = (result +500) / 1000; // scale by 1000 with rounding
					xil_printf ("  Time (seconds): %d \n\r",result);
					}
				}
			break;
		}
		result = (*tTotal * (XTime) 1000000) / (XTime) XPAR_CPU_PPC405_CORE_CLOCK_FREQ_HZ;
		return (result);
	}
