/*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2005 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/
#ifndef XTFT_H /* prevent circular inclusions */
#define XTFT_H /* by using protection macros */

/***************************** Include Files ********************************/

#include "xbasic_types.h"
#include "xstatus.h"

/************************** Constant Definitions ****************************/

/**************************** Type Definitions ******************************/

typedef struct
{
    Xuint32 BaseAddress;
    Xuint32 X;
    Xuint32 Y;
    Xuint32 FgColor;
    Xuint32 BgColor;
    Xuint32 IsReady;
} XTft;

typedef struct
{
    Xuint16 DeviceId;       /* Unique ID  of device */
    Xuint32 BaseAddress;    /* Base address of device */
} XTft_Config;

/***************** Macros (Inline Functions) Definitions ********************/

/************************** Function Prototypes *****************************/

XStatus XTft_Initialize(XTft *InstancePtr, Xuint16 DeviceId);

XTft_Config *XTft_LookupConfig(Xuint16 DeviceId);

XStatus XTft_Write(XTft *InstancePtr, Xint8 val);
XStatus XTft_SetPixel(XTft *InstancePtr, Xuint32 x, Xuint32 y, Xuint32 color);
XStatus XTft_GetPixel(XTft *InstancePtr, Xuint32 x, Xuint32 y, Xuint32 *color);
XStatus XTft_ClearScreen(XTft *InstancePtr);
XStatus XTft_Scroll(XTft *InstancePtr);
XStatus XTft_SetPos(XTft *InstancePtr, Xuint32 x, Xuint32 y);
XStatus XTft_SetColor(XTft *InstancePtr, Xuint32 fgColor, Xuint32 bgColor);

#endif

#ifndef XTFT_CHARCODE_H
#define XTFT_CHARCODE_H

#include "xbasic_types.h"

#define XTFT_CHAR_WIDTH       8
#define XTFT_CHAR_HEIGHT      12

extern Xuint8 XTft_vidChars[256][12];
		
#endif
#include "xbasic_types.h"
#include "xio.h"

#define XTFT_DISPLAY_WIDTH         640
#define XTFT_DISPLAY_HEIGHT        480

#define XTFT_DISPLAY_BUFFER_WIDTH  1024


#define XTft_mClearScreen(BaseAddress, color)     \
         XTft_FillScreen(BaseAddress, 0, 0,      \
                         XTFT_DISPLAY_WIDTH,      \
                         XTFT_DISPLAY_HEIGHT,      \
                         color);

#define XTft_mGetColor(r, g, b)                  \
         ((r) << 16 + (g) << 8 + (b))

#define XTft_mSetPixel(BaseAddress, x, y, color) \
         XIo_Out32((BaseAddress) + \
                   4 * ((y) * XTFT_DISPLAY_BUFFER_WIDTH + x), color)

#define XTft_mGetPixel(BaseAddress, x, y) \
         XIo_In32((BaseAddress) + \
                  4 * ((y) * XTFT_DISPLAY_BUFFER_WIDTH + x))



void XTft_FillScreen(Xuint32 BaseAddress, Xuint32 xu, Xuint32 yu, 
                     Xuint32 xl, Xuint32 yl, Xuint32 color);

void XTft_WriteChar(Xuint32 BaseAddress, Xint8 ch, Xuint32 xu, Xuint32 yu,
                    Xuint32 fgColor, Xuint32 bgColor);

