-- 
-- Copyright (c) 2002-2007 by Impulse Accelerated Technologies, Inc.
-- All rights reserved.
-- 
-- This source file may be used and redistributed without charge
-- subject to the provisions of the IMPULSE ACCELERATED TECHNOLOGIES,
-- INC. REDISTRIBUTABLE IP LICENSE AGREEMENT, and provided that this
-- copyright statement is not removed from the file, and that any
-- derivative work contains this copyright notice.
-- 
library ieee;
use ieee.std_logic_1164.all;

library impulse;
use impulse.apu_if.all;

library apu_mand_v1_00_a;
use apu_mand_v1_00_a.all;


entity apu_mand is
  generic (
    C_FAMILY           : string                    := "virtex4"  );
  port (
    -- apu ports
    apufcminstruction : std_logic_vector(0 to 31);
    apufcminstrvalid : std_logic;
    apufcmradata : std_logic_vector(0 to 31);
    apufcmrbdata : std_logic_vector(0 to 31);
    apufcmoperandvalid : std_logic;
    apufcmflush : std_logic;
    apufcmwritebackok : std_logic;
    apufcmloaddata : std_logic_vector(0 to 31);
    apufcmloaddvalid : std_logic;
    apufcmloadbyteen : std_logic_vector(0 to 3);
    apufcmendian : std_logic;
    apufcmxerca : std_logic;
    apufcmdecoded : std_logic;
    apufcmdecudi : std_logic_vector(0 to 2);
    apufcmdecudivalid : std_logic;
    fcmapuinstrack : out std_logic;
    fcmapuresult : out std_logic_vector(0 to 31);
    fcmapudone : out std_logic;
    fcmapusleepnotready : out std_logic;
    fcmapudecodebusy : out std_logic;
    fcmapudcdgprwrite : out std_logic;
    fcmapudcdraen : out std_logic;
    fcmapudcdrben : out std_logic;
    fcmapudcdprivop : out std_logic;
    fcmapudcdforcealign : out std_logic;
    fcmapudcdxeroven : out std_logic;
    fcmapudcdxercaen : out std_logic;
    fcmapudcdcren : out std_logic;
    fcmapuexecrfield : out std_logic_vector(0 to 2);
    fcmapudcdload : out std_logic;
    fcmapudcdstore : out std_logic;
    fcmapudcdupdate : out std_logic;
    fcmapudcdldstbyte : out std_logic;
    fcmapudcdldsthw : out std_logic;
    fcmapudcdldstwd : out std_logic;
    fcmapudcdldstdw : out std_logic;
    fcmapudcdldstqw : out std_logic;
    fcmapudcdtraple : out std_logic;
    fcmapudcdtrapbe : out std_logic;
    fcmapudcdforcebesteering : out std_logic;
    fcmapudcdfpuop : out std_logic;
    fcmapuexeblockingmco : out std_logic;
    fcmapuexenonblockingmco : out std_logic;
    fcmapuloadwait : out std_logic;
    fcmapuresultvalid : out std_logic;
    fcmapuxerov : out std_logic;
    fcmapuxerca : out std_logic;
    fcmapucr : out std_logic_vector(0 to 3);
    fcmapuexception : out std_logic;
    apu_clk, reset : in std_logic;
    co_clk : in std_logic
    );
end apu_mand;

architecture impl of apu_mand is
  component mand is
    port (
    reset : in std_ulogic;
    sclk : in std_ulogic;
    clk : in std_ulogic;
    p_test_proc_pixel_stream_en : in std_ulogic;
    p_test_proc_pixel_stream_data : out std_ulogic_vector (23 downto 0);
    p_test_proc_pixel_stream_eos : out std_ulogic;
    p_test_proc_pixel_stream_rdy : out std_ulogic;
    p_test_proc_config_stream_en : in std_ulogic;
    p_test_proc_config_stream_eos : in std_ulogic;
    p_test_proc_config_stream_data : in std_ulogic_vector (31 downto 0);
    p_test_proc_config_stream_rdy : out std_ulogic
    );
  end component;

  signal p_test_proc_pixel_stream_rdy : std_ulogic;
  signal p_test_proc_pixel_stream_en : std_ulogic;
  signal p_test_proc_pixel_stream_eos : std_ulogic;
  signal p_test_proc_pixel_stream_idata : std_ulogic_vector (23 downto 0);
  signal p_test_proc_config_stream_rdy : std_ulogic;
  signal p_test_proc_config_stream_en : std_ulogic;
  signal p_test_proc_config_stream_eos : std_ulogic;
  signal p_test_proc_config_stream_idata : std_ulogic_vector (31 downto 0);

  signal p_test_proc_pixel_stream_result : std_logic_vector(0 to 31);
  signal p_test_proc_pixel_stream_done : std_logic;
  signal p_test_proc_pixel_stream_sleepnotready : std_logic;
  signal p_test_proc_pixel_stream_loadwait : std_logic;
  signal p_test_proc_pixel_stream_resultvalid : std_logic;
  signal p_test_proc_pixel_stream_cs : std_logic;
  signal p_test_proc_pixel_stream_instrvalid : std_logic;
  signal p_test_proc_config_stream_result : std_logic_vector(0 to 31);
  signal p_test_proc_config_stream_done : std_logic;
  signal p_test_proc_config_stream_sleepnotready : std_logic;
  signal p_test_proc_config_stream_loadwait : std_logic;
  signal p_test_proc_config_stream_resultvalid : std_logic;
  signal p_test_proc_config_stream_cs : std_logic;
  signal p_test_proc_config_stream_instrvalid : std_logic;

  -- apu signals
  signal apusel : std_logic_vector(0 to 0);
  signal apufcmdecfpuop : std_logic;
  -- end apu signals
begin
apusel <= apufcminstruction(8 to 8);
  apufcmdecfpuop <= '1' when apufcminstruction(0)='1' or apufcminstruction(26)='1' else '0';
  p_test_proc_pixel_stream_cs <= '1' when apusel = "0" else '0';
  p_test_proc_pixel_stream_instrvalid <= '1' when (apufcminstrvalid = '1') and (p_test_proc_pixel_stream_cs = '1') and apufcmdecfpuop = '0' else '0';
  p_test_proc_pixel_stream_if: stream_to_apu
    generic map (
      datawidth => 24
    )
    port map (
      reset, apu_clk,
      p_test_proc_pixel_stream_cs,
      apufcminstruction,
      p_test_proc_pixel_stream_instrvalid,
      apufcmradata,
      apufcmrbdata,
      apufcmoperandvalid,
      apufcmflush,
      apufcmwritebackok,
      apufcmloaddata,
      apufcmloaddvalid,
      apufcmloadbyteen,
      apufcmendian,
      apufcmxerca,
      apufcmdecoded,
      apufcmdecudi,
      apufcmdecudivalid,
      p_test_proc_pixel_stream_result,
      p_test_proc_pixel_stream_done,
      p_test_proc_pixel_stream_sleepnotready,
      p_test_proc_pixel_stream_loadwait,
      p_test_proc_pixel_stream_resultvalid,
      p_test_proc_pixel_stream_en,
      p_test_proc_pixel_stream_rdy,
      p_test_proc_pixel_stream_eos,
      p_test_proc_pixel_stream_idata
    );

  p_test_proc_config_stream_cs <= '1' when apusel = "1" else '0';
  p_test_proc_config_stream_instrvalid <= '1' when (apufcminstrvalid = '1') and (p_test_proc_config_stream_cs = '1') and apufcmdecfpuop = '0' else '0';
  p_test_proc_config_stream_if: apu_to_stream
    generic map (
      datawidth => 32
    )
    port map (
      reset, apu_clk,
      p_test_proc_config_stream_cs,
      apufcminstruction,
      p_test_proc_config_stream_instrvalid,
      apufcmradata,
      apufcmrbdata,
      apufcmoperandvalid,
      apufcmflush,
      apufcmwritebackok,
      apufcmloaddata,
      apufcmloaddvalid,
      apufcmloadbyteen,
      apufcmendian,
      apufcmxerca,
      apufcmdecoded,
      apufcmdecudi,
      apufcmdecudivalid,
      p_test_proc_config_stream_result,
      p_test_proc_config_stream_done,
      p_test_proc_config_stream_sleepnotready,
      p_test_proc_config_stream_loadwait,
      p_test_proc_config_stream_resultvalid,
      p_test_proc_config_stream_en,
      p_test_proc_config_stream_rdy,
      p_test_proc_config_stream_eos,
      p_test_proc_config_stream_idata
    );

  fcmapuresult <= p_test_proc_pixel_stream_result or p_test_proc_config_stream_result;
  fcmapudone <= p_test_proc_pixel_stream_done or p_test_proc_config_stream_done;
  fcmapusleepnotready <= p_test_proc_pixel_stream_sleepnotready or p_test_proc_config_stream_sleepnotready;
  fcmapuloadwait <= p_test_proc_pixel_stream_loadwait or p_test_proc_config_stream_loadwait;
  fcmapuresultvalid <= p_test_proc_pixel_stream_resultvalid or p_test_proc_config_stream_resultvalid;

  -- unused signals
  FCMAPUINSTRACK <= '0';
  FCMAPUDECODEBUSY <= '0';
  FCMAPUDCDGPRWRITE <= '0';
  FCMAPUDCDRAEN <= '0';
  FCMAPUDCDRBEN <= '0';
  FCMAPUDCDPRIVOP <= '0';
  FCMAPUDCDFORCEALIGN <= '0';
  FCMAPUDCDXEROVEN <= '0';
  FCMAPUDCDXERCAEN <= '0';
  FCMAPUDCDCREN <= '0';
  FCMAPUEXECRFIELD <= "000";
  FCMAPUDCDLOAD <= '0';
  FCMAPUDCDSTORE <= '0';
  FCMAPUDCDUPDATE <= '0';
  FCMAPUDCDLDSTBYTE <= '0';
  FCMAPUDCDLDSTHW <= '0';
  FCMAPUDCDLDSTWD <= '0';
  FCMAPUDCDLDSTDW <= '0';
  FCMAPUDCDLDSTQW <= '0';
  FCMAPUDCDTRAPLE <= '0';
  FCMAPUDCDTRAPBE <= '0';
  FCMAPUDCDFORCEBESTEERING <= '0';
  FCMAPUDCDFPUOP <= '0';
  FCMAPUEXEBLOCKINGMCO <= '0';
  FCMAPUEXENONBLOCKINGMCO <= '0';
  FCMAPUXEROV <= '0';
  FCMAPUXERCA <= '0';
  FCMAPUCR <= "0000";
  FCMAPUEXCEPTION <= '0';

  mand_0: mand
    port map (
      reset,
      apu_clk,
      co_clk,
      p_test_proc_pixel_stream_en,
      p_test_proc_pixel_stream_idata,
      p_test_proc_pixel_stream_eos,
      p_test_proc_pixel_stream_rdy,
      p_test_proc_config_stream_en,
      p_test_proc_config_stream_eos,
      p_test_proc_config_stream_idata,
      p_test_proc_config_stream_rdy);

end;

