/******************************************************************************
 *
 *     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 *     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
 *     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
 *     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
 *     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
 *     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 *     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 *     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
 *     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 *     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 *     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 *     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *     FOR A PARTICULAR PURPOSE.
 *     
 *     (c) Copyright 2005 Xilinx, Inc.
 *     All rights reserved.
 *
 ******************************************************************************/

//----------------------------------------------------------------------------
// Filename:		mand.h
// 
// Description:		Constants for Mandelbrot image and Macros are defined
//
// Design Notes:
//   
//-----------------------------------------------------------------------------
// Xilinx APD
// History:
// July 2005 - Created - KRS
//
//-----------------------------------------------------------------------------

// A higher iteration count will provide a sharper image but increase 
// computation time
#define MAX_ITERATIONS 5000

//Size of generated Mandelbrot image in pixels
#define XSIZE 320
#define YSIZE 240
#define IMAGESIZE (XSIZE * YSIZE)

//Number of bits reserved for fractional part of fixed-point number
#define FRACBITS  24

//Change parameter to move to different parts of the Mandelbrot image
#define XL -0.396562500000
#define XH -0.380470703125
#define YL  0.588052734375
//#define YH  0.604144531250

//Fixed point number macros
#define FXCONST(a) FXCONST32(a,FRACBITS)
#define FXADD(a,b) FXADD32(a,b,FRACBITS)
#define FXSUB(a,b) FXSUB32(a,b,FRACBITS)
#define FXMUL(a,b) FXMUL32(a,b,FRACBITS)
#define FXDIV(a,b) FXDIV32(a,b,FRACBITS)
#define FX2REAL(a) FX2REAL32(a,FRACBITS)

// Masks for pixel colors
#define REDMASK 0x0000ff
#define GREENMASK 0x00ff00
#define BLUEMASK 0xff0000

//General purpose macros
#define ABS(n) ((n) < 0 ? -(n) : (n))
#define MAX(n1,n2) ((n1) > (n2) ? (n1) : (n2))
#define MAX4(n1,n2,n3,n4) (MAX(MAX((n1),(n2)),MAX((n3),(n4))))

//required header files for target execution
#ifdef IMPULSE_C_TARGET
	
	//Datatype defines
	#define uint8_t unsigned char
	#define int16_t short int
	#define uint32_t unsigned int
	
	//if executing on FPGA then use xil_printf
	//xil_printf() is smaller then printf(), but does not
	//support real numbers.
	#define printf xil_printf

#endif

